@@ -11,6 +11,8 @@ import View
 import spyre
 
 import cherrypy
+from cherrypy.lib.static import serve_file
+from cherrypy.lib.static import serve_fileobj
 
 ROOT_DIR = os.path.dirname(os.path.realpath(__file__))
 
@@ -20,7 +22,7 @@ templateEnv = jinja2.Environment( loader=templateLoader )
 
 
 class Root(object):
-	def __init__(self,templateVars=None, title=None, inputs=None, outputs=None, controls=None, tabs=None, getJsonDataFunction=None, getDataFunction=None, getPlotFunction=None, getImageFunction=None, getD3Function=None, getCustomCSSFunction=None, getCustomJSFunction=None, getHTMLFunction=None, noOutputFunction=None):
+	def __init__(self,templateVars=None, title=None, inputs=None, outputs=None, controls=None, tabs=None, getJsonDataFunction=None, getDataFunction=None, getPlotFunction=None, getImageFunction=None, getD3Function=None, getCustomCSSFunction=None, getCustomJSFunction=None, getHTMLFunction=None,  getDownloadFunction=None, noOutputFunction=None):
 		# populate template dictionary for creating input,controler, and output HTML and javascript
 		if templateVars is not None:
 			self.templateVars = templateVars
@@ -46,6 +48,7 @@ class Root(object):
 		self.getCustomCSS = getCustomCSSFunction
 		self.getHTML = getHTMLFunction
 		self.noOutput = noOutputFunction
+		self.getDownload = getDownloadFunction
 		d3 = self.getD3()
 		custom_js = self.getCustomJS()
 		custom_css = self.getCustomCSS()
@@ -95,7 +98,7 @@ class Root(object):
 		args = self.clean_args(args)
 		df = self.getData(args)
 		cherrypy.response.headers['Content-Type'] = 'text/html'
-		html = df.to_html(index=False)
+		html = df.to_html(index=False, escape=False)
 		i = 0
 		for col in df.columns:
 			html = html.replace('<th>{}'.format(col),'<th><a onclick="sortTable({},"table0");"><b>{}</b></a>'.format(i,col))
@@ -111,6 +114,18 @@ class Root(object):
 		cherrypy.response.headers['Content-Type'] = 'text/html'
 		return html
 
+	@cherrypy.expose
+	def download(self, **args):
+		args = self.clean_args(args)
+		filepath = self.getDownload(args)
+		print type(filepath).__name__
+		if type(filepath).__name__=="str":
+			return serve_file(filepath, "application/x-download", "attachment", name='data.csv')
+		if type(filepath).__name__=="instance":
+			return serve_fileobj(filepath.getvalue(), "application/x-download", "attachment", name='data.csv')
+		else:
+			return "error downloading file. filepath must be string of buffer"
+
 	@cherrypy.expose
 	def no_output(self, **args):
 		args = self.clean_args(args)
@@ -196,25 +211,31 @@ class App:
 	def getImage(self, params):
 		"""Override this function
 
-		arguments:
-		params (dict)
-
-		returns:
-		matplotlib.image figure
+		arguments: params (dict)
+		returns: matplotlib.image (figure)
 		"""
 		return np.array([[0,0,0]])
 
 	def getHTML(self, params):
 		"""Override this function
 
-		arguments:
-		params (dict)
-
-		returns:
-		html string
+		arguments: params (dict)
+		returns: html (string)
 		"""
 		return "<b>Override</b> the getHTML method to insert your own HTML <i>here</i>"
 
+	def getDownload(self, params):
+		"""Override this function
+
+		arguments: params (dict)
+		returns: path to file or buffer to be downloaded (string or buffer)
+		"""
+		df = self.getData(params)
+		buffer = StringIO.StringIO()
+		df.to_csv(buffer, index=False)
+		filepath = buffer
+		return filepath
+
 	def noOutput(self, params):
 		"""Override this function
 		A method for doing stuff that doesn't reququire an output (refreshing data,
@@ -248,7 +269,7 @@ class App:
 		return ""
 
 	def launch(self,host="local",port=8080):
-		webapp = Root(templateVars=self.templateVars, title=self.title, inputs=self.inputs, outputs=self.outputs, controls=self.controls, tabs=self.tabs, getJsonDataFunction=self.getJsonData, getDataFunction=self.getData, getPlotFunction=self.getPlot, getImageFunction=self.getImage, getD3Function=self.getD3, getCustomJSFunction=self.getCustomJS, getCustomCSSFunction=self.getCustomCSS, getHTMLFunction=self.getHTML, noOutputFunction=self.noOutput)
+		webapp = Root(templateVars=self.templateVars, title=self.title, inputs=self.inputs, outputs=self.outputs, controls=self.controls, tabs=self.tabs, getJsonDataFunction=self.getJsonData, getDataFunction=self.getData, getPlotFunction=self.getPlot, getImageFunction=self.getImage, getD3Function=self.getD3, getCustomJSFunction=self.getCustomJS, getCustomCSSFunction=self.getCustomCSS, getHTMLFunction=self.getHTML, getDownloadFunction=self.getDownload, noOutputFunction=self.noOutput)
 		if host!="local":
 			cherrypy.server.socket_host = '0.0.0.0'
 		cherrypy.server.socket_port = port
